;====================================================================
;Kurts Timer Serial and XBee support functions
;
;Description: This support file will be included by several projects 
;in this directory for building a consistent set of XBEE based function
;that can use the modified DIY Remote control.
;
;Hardware setup: DIY XBee
;
;NEW IN V1.0
;	- First Release
;====================================================================

;[DEBUG]
'DEBUG_OUT con 1
'DEBUG_VERBOSE con 1
'DEBUG_ENTERLEAVE con 1
'DEBUG_DUMP con 1
'DEBUG_WITH_LEDS con 1


#ifdef USEXBEE
#ifndef USETIMER
USETIMER con 1
#endif

#ifndef XBEEONHSERIAL
#ifndef USETASERIAL
USETASERIAL con 1
#endif
#endif

#endif


;====================================================================
; [Private] - Things that will be used only inside this file
;====================================================================
;[Timer]
#ifdef USETIMER
lTimerCnt				var	LONG	; used now also in timing of how long since we received a message
#endif

;--------------------------------------------------------------------
;[DIY XBee Controller Variables]
#ifdef USEXBEE
_bPacketNum			var	byte		; A packet number...
_bPacketHeader		var	byte(4)		; Have a 4 byte header to read in first
_bTemp				var	byte(10)	; for reading in different strings...
_b
_bChkSumIn			var	byte
_lCurrentTimeT		var	long		; 
_lTimerLastPacket	var	long		; the timer value when we received the last packet
_lTimerLastRequest	var	long		; what was the timer when we last requested data?
_lTimeDiffMS		var long		; calculated time difference

_fNewPacketAvail	var	bit			; Is there a new packet available?
_fPacketValidPrev	var	bit			; The previous valid...
_fReqDataPacketSent	var	bit			; has a request been sent since we last received one?
_fReqDataForced		var	bit			; Was our request forced by a timeout?
_tas_i				var	byte
_tas_b				var	byte		; 
_cbRead				var	byte		; how many bytes did we read?
;----------------- Other variables --------------------------------------------


; These pins are defined if they were not previously defined.  This allows projects
; to redefine them.  Be careful with RTS if using sparkfun regulated explroer as
; the RTS line is not voltage shifted on these adapters.

#ifndef cXBEE_IN
cXBEE_IN				con	p14		; The input pin
#endif

#ifndef cXBEE_OUT
cXBEE_OUT				con	p15		; the output pin
#endif

#ifndef XBEEONHSERIAL
#ifndef CXBEE_RTS   
CXBEE_RTS				con	p7
#endif
#endif

#ifndef CXBEE_BAUD
#ifdef XBEEONHSERIAL
CXBEE_BAUD				con H62500				; Non-standard baud rate for xbee but...
#else
cXBEE_BAUD				con i57600				; now using standard serial
;cXBEE_BAUD				con	0x4020	; i62500	; BUGBUG - not standard! - Is for HSERIAL
#endif	; XBEEONHSERIAL

#endif	; CXBEE_BAUD

; Also define some timeouts.  Allow users to override

#ifndef CXBEEPACKETTIMEOUTMS
CXBEEPACKETTIMEOUTMS con 500					; how long to wait for packet after we send request
#endif

#ifndef CXBEEFORCEREQMS	
CXBEEFORCEREQMS		con	1000					; if nothing in 1 second force a request...
#endif

#ifndef CXBEETIMEOUTRECVMS
CXBEETIMEOUTRECVMS	con	2000					; 2 seconds if we receive nothing
#endif
;==============================================================================
; If XBEE is on HSERIAL, then define some more stuff...
;==============================================================================
#ifdef XBEEONHSERIAL
ENABLEHSERIAL
HSERSTAT_CLEAR_INPUT	con 0 			;Clear input buffer
HSERSTAT_CLEAR_OUTPUT	con	1 			;Clear output buffer
HSERSTAT_CLEAR_BOTH		con	2 			;Clear both buffers
HSERSTAT_INPUT_DATA		con	3 			;If input data is available go to label
HSERSTAT_INPUT_EMPTY	con	4 			;If input data is not available go to label
HSERSTAT_OUTPUT_DATA	con	5 			;If output data is being sent go to label
HSERSTAT_OUTPUT_EMPTY	con	6 			;If output data is not being sent go to label
bHSerinHasData			var	byte		; 
#endif

#endif ; USE_XBEE


; TASerial definitions
_TASI_cFO	var	byte			; how many bytes have we cached away in our serial buffer

;==============================================================================
; Timer - Support
;==============================================================================
;==============================================================================
; [InitTimer] - Initialize the timer for our timer tics and the like
;==============================================================================
#ifdef USETIMER

InitTimer:
#ifdef DEBUG_OUT
	serout s_out, i9600, ["Init Timer", 13]
#endif	
	; 
	; Timer A init, used for timing of messages and some times for timing code...
	TMA = 0	; clock / 8192					; Low resolution clock - used for timeouts...
	lTimerCnt = 0
	ONASMINTERRUPT TIMERAINT, HANDLE_TIMERA_ASM 
	ENABLE TIMERAINT
	return
;==============================================================================
;[Handle_Timer_asm] - Handle timer A overlfow in assembly language.  Currently only
;used for timings for debuging the speed of the code
;Now used to time how long since we received a message from the remote.
;this is important when we are in the NEW message mode, as we could be hung
;out with the robot walking and no new commands coming in.
;==============================================================================

   BEGINASMSUB 
HANDLE_TIMERA_ASM 
	push.l 	er1                  ; first save away ER1 as we will mess with it. 
	bclr 	#6,@IRR1:8               ; clear the cooresponding bit in the interrupt pending mask 
	mov.l 	@LTIMERCNT:16,er1      ; Add 256 to our counter 
	add.l	#256,er1 
	mov.l 	er1, @LTIMERCNT:16 
	pop.l 	er1 
	rte 
	ENDASMSUB 

;--------------------------------------------------------------------
;[GetCurrentTime] - Gets the Timer value from our overflow counter as well as the TCA counter.  It
;                makes sure of consistancy. That is it is very posible that 
;                after we grabed the timers value it overflows, before we grab the other part
;                so we check to make sure it is correct and if necesary regrab things.
;==============================================================================
	return		; Put a basic statement before...
lCurrentTime			var	long
	
GetCurrentTime:
	lCurrentTime = lTimerCnt + TCA
	
	; handle wrap
	if lTimerCnt <> (lCurrentTime & 0xffffff00) then
		lCurrentTime = lTimerCnt + TCA
	endif

	return lCurrentTime

	
#endif ; USETIMER

;==============================================================================
; XBEE - Support
;==============================================================================
#ifdef USEXBEE

#ifdef XBEEONHSERIAL

;==============================================================================
; [WaitHSeroutComplete] - This simple helper function waits until the HSersial
; 							output buffer is empty before continuing...
; BUGBUG: Rolling our own as the hserstat reset the processor
;==============================================================================
WaitHseroutComplete:
	nop						; transisition to assembly language.
_WTC_LOOP:	
	mov.b	@_HSEROUTSTART,r0l
	mov.b	@_HSEROUTEND,r0h
	cmp.b	r0l,r0h
	bne		_WTC_LOOP:8

	;hserstat 5, WaitTransmitComplete				; wait for all output to go out...	
	return
#endif	; XBEEONHSERIAL



;==============================================================================
; [InitXbee] - Initialize the XBEE for use with DIY support
;==============================================================================
InitXBee:
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Enter: Init Controller", 13]
#endif	

	pause 20							; have to have a guard time to get into Command sequence
#ifdef XBEEONHSERIAL
	sethserial cXBEE_BAUD, H8DATABITS,HNOPARITY,H1STOPBITS
#ifdef cXBEE_RTS			; RTS line optional for us with HSERIAL 
	hserout [str _XBEE_PPP_STR\3]
	gosub WaitHSeroutComplete
	pause 20							; have to wait a bit again
	hserout [	"ATD6 1",13,|			; turn on flow control
				str _XBEE_ATCN_STR\5]				; exit command mode
	low cXBEE_RTS						; make sure RTS is output and input starts off enabled...

#endif ; cXBEE_RTS

#else
	; We assume we have a control pin.	
	serout cXBEE_OUT, cXBEE_BAUD, [str _XBEE_PPP_STR\3]
	pause 20							; have to wait a bit again
	serout cXBEE_OUT, cXBEE_BAUD, [	"ATD6 1",13,|			; turn on flow control
				str _XBEE_ATCN_STR\5]				; exit command mode
#endif
	pause 10	; need to wait to exit command mode...

	cPacketTimeouts = 0
	fTransReadyRecvd = 0
	_fPacketValidPrev = 0
	fPacketEnterSSCMode = 0   
	_fReqDataPacketSent = 0
	_fReqDataForced = 0			;
    gosub ClearInputBuffer		; make sure we toss everything out of our buffer and init the RTS pin

#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Exit: Init Controller", 13]
#endif	
 
return

;==============================================================================
; [XBeeOutputVal[bXbeeVal]] - Output a value back to the device.  Works for XBee
; 		by sending Byte value back to the DIY remote to display.
; Parmameters:
;		bXbeeVal - Value to send back
;==============================================================================
bXbeeVal	var	byte
XBeeOutputVal[bXbeeVal]:
	gosub SendXBeePacket[XBEE_RECV_DISP_VAL, bXbeeVal, 0, 0]		
return

;==============================================================================
; [XBeeOutputString] - Output a string back to the device.  Works for XBee
; 		by outputing the string back to the DIY remote to display.
; Parmameters:
;		pString - Pointer to string.  First byte contains the length
;==============================================================================
pString	var	pointer
XBeeOutputString[pString]:
	; BUGBUG:: Will not work for zero count, but what the heck.
	; First we need to get the count a compute the checksum - for the heck of it will use assembly...
	mov.l	@PSTRING:16, er0		; get the pointer value into ero
	mov.b	@er0+, r1l				; Get the count into R1l - start of the checksum
	mov.l	er0, @PSTRING:16		; Update pointer value to be after the count byte.
	mov.b	r1l, @_TAS_B:16			; save away count to pass to the serial output...

	gosub SendXBeePacket[XBEE_RECV_DISP_VAL, 0,  _tas_b, pString]		; Send Data to remote (CmdType, ChkSum, Packet Number, CB extra data)


#ifdef DEBUG_OUT
	serout s_out, i9600, ["XOS: ", dec _tas_b, ":", str @pString\_tas_b, 13]
#endif

return

;==============================================================================
; [XBeePlaySounds] - Sends the buffer of souncs back to the remote to play
; Parmameters:
;		pSnds - Pointer to the sounds
;		cbSnds - Size of the buffer
;==============================================================================
; Too bad I don't have real macros to save space...
pSnds 	var pointer
cbSnds	var	byte
XBeePlaySounds[pSnds, cbSnds]:
	gosub SendXBeePacket[XBEE_PLAY_SOUND, 0,  cbSnds, pSnds]		; Send Data to remote (CmdType, ChkSum, Packet Number, CB extra data)
	return
	
;==============================================================================
; [XbeeRTSAllowInput] - This function enables or disables the RTS line for the XBEE
; This is only used for the XBEE on HSERIAL mode as the TASerial has the RTS stuff
; built in and we don't want to throw away characters...
;==============================================================================
fRTSEnable	var	byte
XbeeRTSAllowInput[fRTSEnable]
#ifdef XBEEONHSERIAL
#ifdef cXBEE_RTS
	if fRTSEnable then
		low cXBEE_RTS
	else
		high cXBEE_RTS
	endif
#endif
#endif
return
;==============================================================================
; [SetXbeeDL] - Set the XBee DL to the specified word that is passed
; BUGBUG: sample function, need to clean up.
;==============================================================================
wNewDL	var	word
SetXBeeDL[wNewDL]:
#ifdef DEBUG_VERBOSE
	;bugbug;;; debug
	serout s_out, i9600, ["Set XBEE DL: ", hex wNewDL, 13]
#endif

#ifdef XBEEONHSERIAL
	pause 20							; have to have a guard time to get into Command sequence
	hserout [str _XBEE_PPP_STR\3]
	gosub WaitHSeroutComplete
	pause 20							; have to wait a bit again

	hserout ["ATDL ",hex wNewDL, 13, |	; Set the New DL
			 str _XBEE_ATCN_STR\5]					; Exit command mode
  	pause 10
#else
	pause 20							; have to have a guard time to get into Command sequence
	serout cXBEE_OUT, cXBEE_BAUD, [str _XBEE_PPP_STR\3]
	pause 20							; have to wait a bit again

	serout cXBEE_OUT, cXBEE_BAUD,["ATDL ",hex wNewDL, 13, |	; Set the New DL
			 str _XBEE_ATCN_STR\5]					; Exit command mode
  	pause 10
#endif
	return

;==============================================================================
; [GetXbeeDL] - Retrieve the XBee DL and return it as a Word
;==============================================================================
GetXBeeDL:
	gosub GetXBeeHexVal["D","L",0x3], wNewDL
	return wNewDL


;==============================================================================
; [GetXbeeHexVal] - Retrieves one of the XBee values that return some hex value
;	this includes: DL, MY, SH, SL...
;	_c1 and _c2 are the two bytes of the command for my "M", "Y"
;==============================================================================
_c1			var	byte	; First char of command name "M"
_c2 		var byte	; 2nd char of Command        "Y"
_SSAction 	var byte 	; Two bits low says .bit0 = Enter command mode .bit1 exit command mode
						; so we can chain up a few of these if we want...
_lVal		var	long						
GetXBeeHexVal[_c1, _c2, _SSAction]:

#ifdef DEBUG_VERBOSE
	;bugbug;;; debug
	serout s_out, i9600, ["Get XBEE HEX VaL: (", _c1, _c2, ")",13]
#endif
	_lVal = -1	; error condition

	if _SSAction.bit0 then
		gosub ClearInputBuffer
#ifdef XBEEONHSERIAL
		pause 20							; have to have a guard time to get into Command sequence
		hserout [str _XBEE_PPP_STR\3]
		gosub WaitHSeroutComplete
		pause 20							; have to wait a bit again

		; need to process the OK for the +++
		HSERIN 20000, _GXHV_TO,	[str _bTemp\10\13]	; first OK for +++
	endif
	
	; Now output our command	
	hserout ["AT", _c1, _c2,13]

	; Get our value now
	HSERIN 20000, _GXHV_TO, [hex _lVal]		; get the value
	
	if _SSAction.bit1 then
		hserout [str _XBEE_ATCN_STR\5]							; Exit command mode
		HSERIN 20000, _GXHV_TO,	[str _bTemp\10\13]	; and retrieve the OK
	endif
_GXHV_TO:	

#else
		pause 20							; have to have a guard time to get into Command sequence
		serout cXBEE_OUT, cXBEE_BAUD, [str _XBEE_PPP_STR\3]
		pause 20							; have to wait a bit again

		gosub TASerin[cXBEE_IN, cXBEE_RTS, cXBEE_BAUD, @_bTemp, 10, 20000, 13], _cbRead
		if _cbRead = 0 then
			return 0xffff	; return an error condition
		endif
	endif

	; OK lets output our command
	serout cXBEE_OUT, cXBEE_BAUD,["AT", _c1, _c2, 13]

	; And lets get the repsonse and convert the hex value
	gosub TASerin[cXBEE_IN, cXBEE_RTS, cXBEE_BAUD, @_bTemp, 10, 20000, 13], _cbRead
	if _cbRead < 2 then
			return 0xffff	; return an error condition
	endif
	_lVal = 0
	_cbRead = _cbRead-2
	for _tas_i = 0 to _cbRead	; we don't need to process the <cr> 
		if _bTemp(_tas_i) >= "0" and  _bTemp(_tas_i) <= "9" then
			_lVal = _lVal * 16 + _bTemp(_tas_i) - "0"
		elseif _bTemp(_tas_i) >= "a" and  _bTemp(_tas_i) <= "f"
			_lVal = _lVal * 16 + _bTemp(_tas_i) - "a"
		elseif _bTemp(_tas_i) >= "A" and  _bTemp(_tas_i) <= "F" 
			_lVal = _lVal * 16 + _bTemp(_tas_i) - "A"
		endif
	next

	if _SSAction.bit1 then
		serout cXBEE_OUT, cXBEE_BAUD,[str _XBEE_ATCN_STR\5]					; Exit command mode

		; retrieve the OK that returned
		gosub TASerin[cXBEE_IN, cXBEE_RTS, cXBEE_BAUD, @_bTemp, 10, 20000, 13], _cbRead
	endif
#endif	
	return _lVal


;==============================================================================
; [GetXbeeStringVal] - Retrieves one of the XBee values that return some string
;	this includes: NI
;	_c1 and _c2 are the two bytes of the command for my "M", "Y"
;==============================================================================
_pbSVal		var	pointer
		Sound P9,[100\4400]
_pbT		var	pointer
_cT			var	byte
_cbSVal		var	byte
_GXSV_cbRet var byte
_GXSV_i		var	byte
GetXBeeStringVal[_c1, _c2, _SSAction, _pbSVal, _cbSVal]:

#ifdef DEBUG_VERBOSE
	;bugbug;;; debug
	serout s_out, i9600, ["GXBEE SVaL: (", _c1, _c2, ")",13]
#endif
	_GXSV_cbRet = 0
	
	if _SSAction.bit0 then
		gosub ClearInputBuffer
#ifdef XBEEONHSERIAL
		pause 20							; have to have a guard time to get into Command sequence
		hserout [str _XBEE_PPP_STR\3]
		gosub WaitHSeroutComplete
		pause 20							; have to wait a bit again

		; need to process the OK for the +++
		HSERIN 20000, _GXSV_TO,	[str _bTemp\10\13]	; first OK for 
	endif
	
	; Now output our command	
	hserout ["AT", _c1, _c2,13]

	; Get our value now
	; BUGBUG: Make sure the pointer passed in is to a byte level...
	_pbsVal.highword = 0x2		
	HSERIN 20000, _GXSV_TO, [str @_pbSVal\_cbSVal\13]		; get the value
	
	; Need to see if we read all of the value or not, ie did we get a CR
	_pbT = _pbSVal
	_cT = _cbSVal - 1

	for _GXSV_cbRet = 0 to _cT			
		if @_pbT = 13 then _GXSV_CR		; Our count should be OK at this point-
		_pbT = _pbT + 1
	next

	; if we got to here then we read the whole thing.
	_GXSV_cbRet = _cbSVal;
	
_GXSV_SCAN_READ_UNTIL_CR:
	HSERIN 20000, _GXSV_TO, [str _bTemp\10\13]	; retrieve rest of string
	for _GXSV_i = 0 to 9
		if (_bTemp(_GXSV_i) = 13) then _GXSV_CR
	next
	; Need to read some more until we hit CR
	goto _GXSV_SCAN_READ_UNTIL_CR
	
_GXSV_CR:	
	if _SSAction.bit1 then
		hserout [str _XBEE_ATCN_STR\5]							; Exit command mode
		HSERIN 20000, _GXSV_TO,	[str _bTemp\10\13]	; and retrieve the OK
	endif
_GXSV_TO:	

#else
		pause 20							; have to have a guard time to get into Command sequence
		serout cXBEE_OUT, cXBEE_BAUD, [str _XBEE_PPP_STR\3]
		pause 20							; have to wait a bit again

		gosub TASerin[cXBEE_IN, cXBEE_RTS, cXBEE_BAUD, @_bTemp, 10, 20000, 13], _cbRead
		if _cbRead = 0 then
			return 0xffff	; return an error condition
		endif
	endif

	; OK lets output our command
	serout cXBEE_OUT, cXBEE_BAUD,["AT", _c1, _c2, 13]

	; And lets get the repsonse and convert the hex value
	gosub TASerin[cXBEE_IN, cXBEE_RTS, cXBEE_BAUD, _pbSVal,  _cbSVal, 20000, 13], _GXSV_cbRet 
	
	if (_GXSV_cbRet = _cbSVal) and (@_pbsVal(_GXSV_cbRet-1) <> 13) then
		; We did not get to the end of the line
		do 
			gosub TASerin[cXBEE_IN, cXBEE_RTS, cXBEE_BAUD, @_bTemp, 10, 20000, 13], _cbRead
		while ((_cbRead = 10) and (_bTemp(9) = 13))
	elseif (_GXSV_cbRet <> _cbSVal)
		_GXSV_cbRet = _GXSV_cbRet -1		; don't return the CR
	endif

	if _SSAction.bit1 then
		serout cXBEE_OUT, cXBEE_BAUD,[str _XBEE_ATCN_STR\5]					; Exit command mode

		; retrieve the OK that returned
		gosub TASerin[cXBEE_IN, cXBEE_RTS, cXBEE_BAUD, @_bTemp, 10, 20000, 13], _cbRead
	endif
#endif	
	return _GXSV_cbRet
	
;==============================================================================
; [SetXBeeMy] - Set the XBee My Value to the passed in value.
; BUGBUG:: Could combine with other functions...
;
;==============================================================================
SetXBeeMy[wNewDL]:

#ifdef DEBUG
	serout s_out, i9600, ["Set XBee My: ", hex wNewDL, 13]
#endif		
#ifdef XBEEONHSERIAL
	pause 20							; have to have a guard time to get into Command sequence
	hserout [str _XBEE_PPP_STR\3]
	gosub WaitHSeroutComplete
	pause 20							; have to wait a bit again
	hserout  ["ATMY ",hex wNewDL, 13, |	; Set the destination address
			str _XBEE_ATCN_STR\5]					; exit command mode
#else
	pause 20							; have to have a guard time to get into Command sequence
	serout cXBEE_OUT, cXBEE_BAUD, [str _XBEE_PPP_STR\3]
	pause 20							; have to wait a bit again
	serout cXBEE_OUT, cXBEE_BAUD, ["ATMY ",hex wNewDL, 13, |	; Set the destination address
			str _XBEE_ATCN_STR\5]					; exit command mode
#endif
	return


;==============================================================================
; [SerinFC] - Simple helper function to use the flow control version of serin
;	to read data from the XBEE.  The only complication is that once we read
;	the number of bytes we want, the flow control is raised back up, but the
;	XBEE may still send 1 or 2 characters.  So buffer these up for the next
;	read...
;==============================================================================
lTimeout	var	long		; how long of a timeout...
pbIN		var	pointer		; pointer to data to retrieve
cbIN		var	word		; count of how many bytes to read in.
abSlop		var	byte(2)		; slop bytes after main serin finished...
cbT			var	word

#ifndef XBEEONHSERIAL
SerinFC[pbIN, cbIN, lTimeout]:
	gosub TASerin[cXBEE_IN, cXBEE_RTS, cXBEE_BAUD, pbIN,  cbIN, lTimeout, -1], cbT
	return cbT
#endif	; USETASERIAL

;==============================================================================
; [SendXBeePacket] - Simple helper function to send the 4 byte packet header
;	 plus the extra data if any
; 	 gosub SendXBeePacket[bPacketType, bSeq, cbExtra, pExtra]
;==============================================================================
#ifdef DEBUG_OUT
_pbDump var pointer
#endif

_bPHType 	var _bPacketHeader(0)
_bPHChkSum	var	_bPacketHeader(1)
_bPHSeq		var	_bPacketHeader(2)
_bPHCBExtra var _bPacketHeader(3)
SendXbeePacket[_bPHType,_bPHSeq,_bPHCBExtra, pbIN]:
	_bPHChkSum = _bPHSeq + _bPHCBExtra	;
	
	; need to finish checksum - could do in basic but need to verify pointer...
	mov.b	@_BPHCHKSUM:16, r1l	; get the checkum already calculated for header
	mov.b	@_BPHCBEXTRA:16, r1h	; get the count of bytes 
	beq		_SXBP_NOEXTR:8				; no extra bytes
	mov.l	@PBIN, er0					; get the pointer to extra data
_SXBP_CHKSUM_LOOP:
	mov.b	@er0+, r2l				; get the next character
	add.b	r2l,  r1l				; add on to r0l for checksum
	dec.b	r1h						; decrement	our counter
	bne		_SXBP_CHKSUM_LOOP:8		; not done yet.  
_SXBP_NOEXTR:
	mov.b	r1l, @_BPHCHKSUM:16		; save away checksum for basic to use


#ifdef XBEEONHSERIAL
	if _bPHCBExtra then
		hserout [str _bPacketHeader\4, str @pbIN\_bPHCBExtra]		; Real simple message 
	else
		hserout [str _bPacketHeader\4]		; Real simple message 
	endif
#else
	if _bPHCBExtra then
		serout cXBEE_OUT, cXBEE_BAUD, [str _bPacketHeader\4, str @pbIN\_bPHCBExtra]		; Real simple message 
	else
		serout cXBEE_OUT, cXBEE_BAUD, [str _bPacketHeader\4]		; Real simple message 
	endif
#endif

#ifdef DEBUG_OUT
	; We moved dump before the serout as hserout will cause lots of interrupts which will screw up our serial output...
	; Moved after as we want the other side to get it as quick as possible...
	_pbDump = pbIN
	_pbDump.highword = 0x2	; BUGBUG: force for pointer to bytes
	serout s_out, i9600, ["SDP:", hex _bPacketHeader(0)\2, hex _bPacketHeader(1)\2, hex _bPacketHeader(2)\2, hex _bPacketHeader(3)\2, ":"]
#ifdef DEBUG_VERBOSE		; Only ouput whole thing if verbose...
	if _bPHCBExtra then
		for _tas_i = 0 to _bPHCBExtra -1
			serout s_out, i9600, [hex @_pbDump\2]
			_pbDump = _pbDump + 1
		next
	endif
#endif	
	serout s_out, i9600, [13]
		
#endif

	
	return
	
;==============================================================================
; [ClearInputBuffer] - This simple helper function will clear out the input
;						buffer from the XBEE
;==============================================================================
ClearInputBuffer:

; 	warning this function does not handle RTS for HSERIAL
; 	assumes that it has been setup properly before it got here.
#ifdef XBEEONHSERIAL
_CIB_LOOP:	
	hserin 1000, _CIB_TO, [_tas_b]
	goto _CIB_LOOP
	
_CIB_TO:
#else
	low cXBEE_RTS		; Ok enable input from the XBEE
_CIB_LOOP:	
	serin cXBEE_IN, cXBEE_BAUD, 1000, _CIB_TO, [_tas_b]
	goto _CIB_LOOP
	
_CIB_TO:
	high cXBEE_RTS		; We only want input when we want it...
	_TASI_cFO = 0			; make sure we don't use any slop bytes for the next read...
#endif
	return


#ifndef DIY_TRANSMITTER
;--------------------------------------------------------------------
;[XBeeResetPacketTimeout] - This function resets the save timer value that is used to 
;				 decide if our XBEE has timed out It could/should be simplay
;				 a call to GetCurrentTime, but try to save a little time of
;				 not nesting the calls...
;==============================================================================
	
XBeeResetPacketTimeout:
	_lTimerLastPacket = lTimerCnt + TCA
	
	; handle wrap
	if lTimerCnt <> (_lTimerLastPacket & 0xffffff00) then
		_lTimerLastPacket = lTimerCnt + TCA
	endif

	return



;==============================================================================
; [ReceiveXBeePacket] - This function will try to receive a packet of information
; 		from the remote control over XBee.
;
; the data in a standard packet is arranged in the following byte order:
;	0 - Buttons High
;	1 - Buttons Low
; 	2 - Right Joystick L/R
;	3 - Right Joystick U/D
;	4 - Left Joystick L/R
;	5 - Left Joystick U/D
; 	6 - Right Slider
;	7 - Left Slider
;==============================================================================

; Thse are variables needed here and not by tranmitter who deoes not include these functions
; BUGBUG:: Defining three variables next to each other does not necessarily imply they will
; be defined consecutively.  So try a different route.
; are sent in a packet...
_alNIPD		var long(7)		; 2 for SL+H + 20 bytes for NI data...
_lSNH		var _alNIPD(0)	; Serial Number High
_lSNL		var _alNIPD(1)	; Serial number low.
_bNI		var	_alNIPD(2)	; String to read NI into...
_pbNI		var	pointer		; need something to point as a byte pointer


ReceiveXBeePacket:
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["E: ReceiveXbeePacket", 13]
#endif	
	_fPacketValidPrev = fPacketValid		; Save away the previous state as this is the state if no new data...
	fPacketValid = 0
	fPacketTimeOut = 0
	_fNewPacketAvail = 0;
	fPacketForced = 0;
	fPacketEnterSSCMode = 0   
	;	We will first see if we have a packet header waiting for us.
#ifdef XBEEONHSERIAL
#ifdef CXBEE_RTS
	low cXBEE_RTS		; Ok enable input from the XBEE - wont turn off by default
	; bugbug should maybe check to see if it was high first and if
	; so maybe bypass the next check...
#endif	
;	hserstat HSERSTAT_INPUT_EMPTY, _TP_Timeout			; if no input available quickly jump out.
	; Well Hserstat is failing, try rolling our own.
	mov.b	@_HSERINSTART, r0l
	mov.b	@_HSERINEND, r0h
	sub.b	r0h, r0l
	mov.b	r0l, @BHSERINHASDATA
	if	(not bHSerinHasData) then _RXP_CHECKFORHEADER_TO
	hserin 10000, _RXP_HEADER_TO, [str _bPacketHeader\4]
	goto _RXP_CHECK_PACKET
_RXP_HEADER_TO:
#ifdef DEBUG_OUT
	serout s_out, i9600, ["TO: - Packet header", 13]
#endif	
	
_RXP_CHECK_PACKET:
#else
	; using our own serin with flow control
	gosub SerinFC[@_bPacketHeader, 4, 10000], _cbRead
	if _cbRead = 0 then _RXP_CHECKFORHEADER_TO
#endif

#ifdef DEBUG_VERBOSE	
	;bugbug;;; debug
	serout s_out, i9600, ["PH:", hex _bPacketHeader(0)\2, hex _bPacketHeader(1)\2, hex _bPacketHeader(2)\2, hex _bPacketHeader(3)\2, 13]
#endif
	
	;-----------------------------------------------------------------------------
	; [XBEE_TRANS_DATA]
	;-----------------------------------------------------------------------------
	; process first as higher number of these come in...
	if (_bPacketHeader(0) = XBEE_TRANS_DATA) and (_bPacketHeader(3) = XBEEPACKETSIZE) then
#ifdef XBEEONHSERIAL
		hserin 25000, _RXP_TIMEOUT, [str bPacket\XBEEPACKETSIZE]
#else
		gosub SerinFC[@bPacket, XBEEPACKETSIZE, 25000], _cbRead
		if _cbRead = 0 then _RXP_TIMEOUT
#endif
		_fReqDataPacketSent = 0	; if we received a packet reset so we will request data again...
		; validate the header information...
		; Ok Lets try to validate the checksum.
		_bChkSumIn = _bPacketHeader(2) + _bPacketHeader(3)
		for _tas_i = 0 to XBEEPACKETSIZE-1
			_bChkSumIn = _bChkSumIn + bPacket(_tas_i)
		next
#ifdef DEBUG_VERBOSE
		serout s_out, i9600, ["P: ", hex bPacket(PKT_BTNLOW)\2, hex bPacket(PKT_BTNHI)\2, hex bPacket(PKT_RJOYLR)\2, hex bPacket(PKT_RJOYUD)\2, |
								     hex bPacket(PKT_LJOYLR)\2, hex bPacket(PKT_LJOYUD)\2, hex bPacket(PKT_RSLIDER)\2, hex bPacket(PKT_LSLIDER)\2, |
								     " CS: ", hex _bChkSumIn, " PN: ", hex _bPacketNum, 13]
#endif		
		if (_bChkSumIn = _bPacketHeader(1)) then
#ifdef DEBUG_OUT
	;bugbug;;; debug
	serout s_out, i9600, ["RV:", hex _bPacketHeader(0)\2, hex _bPacketHeader(1)\2, hex _bPacketHeader(2)\2, hex _bPacketHeader(3)\2, 13]
#endif

			fPacketValid = 1	; data is valid
			cPacketTimeouts = 0	; reset when we have a valid packet
			fPacketForced = _fReqDataForced	; Was the last request forced???
			_fReqDataForced = 0				; clear that state now
			GOSUB XBeeResetPacketTimeout ; sets _lTimerLastPacket
;			toggle p4
			return	;  		; get out quick!
		else
#ifdef DEBUG_OUT
			;bugbug;;; debug
			serout s_out, i9600, ["E ChkSum: ", hex _bPacketHeader(0)\2, hex _bPacketHeader(1)\2, hex _bPacketHeader(2)\2, hex _bPacketHeader(3)\2,  ":", hex _bChkSumIn, 13]
#endif
			; the checksum and data not right lets clear our input buffer out...	
;			toggle p6			; BUGBUG - Debug
			gosub ClearInputBuffer
		endif
	; OK we got 4 bytes is it a proper header and if so what?
	; Currently we are supporting a few different messages including XBEE_TRANS_READY, XBEE_TRANS_READY and XBEE_TRANS_NEW message
	;-----------------------------------------------------------------------------
	; [XBEE_TRANS_READY]
	;-----------------------------------------------------------------------------
	elseif (_bPacketHeader(0) = XBEE_TRANS_READY)
		; Two cases.  everything zero which is the old simple all zero, newer one has the MY of the transmitter, so we will want to update
		; our DL to point to it...
		if  (_bPacketHeader(1) = 0)  and (_bPacketHeader(2) = 0) and (_bPacketHeader(3) = 0) then
			fTransReadyRecvd = 1		; OK we have received a packet saying transmitter is ready.	
#ifdef DEBUG_OUT
			serout s_out, i9600, ["Ready:  ", 13]
#endif
		elseif (_bPacketHeader(3) = 2)
			; we need to read in a new DL for the transmitter...
#ifdef XBEEONHSERIAL
			hserin 10000, _RXP_CHECKFORHEADER_TO, [str wNewDL\2]
#else
			gosub SerinFC[@wNewDL, 2, 10000], _cbRead
#ifdef DEBUG_VERBOSE
			serout s_out, i9600, ["Ready Extra (", dec _cbRead, "):", hex wNewDL, 13]
#endif
			if _cbRead = 0 then _RXP_CHECKFORHEADER_TO
#endif			
#ifdef DEBUG_OUT
			serout s_out, i9600, ["Ready:  ", hex wNewDL, 13]
#endif
			if _bPacketHeader(1) = ((_bPacketHeader(2) + _bPacketHeader(3) + wNewDL.Lowbyte + wNewDl.Highbyte) & 0xff) then ; did on fly make sure byte
				fTransReadyRecvd = 1		; OK we have received a packet saying transmitter is ready.	
				gosub SetXBeeDL[wNewDL]			
				gosub ClearInputBuffer		; get rid of all of the OKs...
			endif
		endif
		Sound P9,[100\4400]

		; And tell the remote to go into New data only mode.
		gosub SendXbeeNewDataOnlyPacket[1]
		GOSUB XBeeResetPacketTimeout ; sets _lTimerLastPacket
		_fReqDataPacketSent = 0							; make sure we don't think we have any outstanding requests
	;-----------------------------------------------------------------------------
	; [XBEE_TRANS_NOTREADY]
	;-----------------------------------------------------------------------------
	elseif (_bPacketHeader(0) = XBEE_TRANS_NOTREADY) and (_bPacketHeader(1) = 0)  and (_bPacketHeader(2) = 0) and (_bPacketHeader(3) = 0)
		; we are being told that the transmitter may not be valid anymore...
		Sound P9,[60\3200]
		fTransReadyRecvd = 0			; Ok not valid anymore...
#ifdef DEBUG_OUT
		serout s_out, i9600, ["Not Ready:", 13]
#endif
		
	;-----------------------------------------------------------------------------
	; [XBEE_TRANS_NEW]
	;-----------------------------------------------------------------------------
	elseif (_bPacketHeader(0) = XBEE_TRANS_NEW) and (_bPacketHeader(1) = 0)  and (_bPacketHeader(2) = 0) and (_bPacketHeader(3) = 0)
		; The remote has told us it has new data available for us, so we should now ask for it!
#ifdef DEBUG_OUT
			serout s_out, i9600, ["New data Packet received ", 13]
#endif
		_fNewPacketAvail = 1;
	;-----------------------------------------------------------------------------
	; [XBEE_REQ_SN_NI]
	;-----------------------------------------------------------------------------
	elseif (_bPacketHeader(0) = XBEE_REQ_SN_NI)
		; The caller must pass through the DL to talk back to, or how else would we???
		; our DL to point to it...
		if (_bPacketHeader(3) = 2) then
			; we need to read in a new DL for the transmitter...
#ifdef XBEEONHSERIAL
			hserin 10000, _RXP_CHECKFORHEADER_TO, [str wNewDL\2]
#else
			gosub SerinFC[@wNewDL, 2, 10000], _cbRead
			if _cbRead = 0 then _RXP_CHECKFORHEADER_TO
#endif			
#ifdef DEBUG_OUT
			serout s_out, i9600, ["XBEE_REQ_SN_NI:  ", hex wNewDL, 13]
#endif
			if _bPacketHeader(1) = ((_bPacketHeader(2) + _bPacketHeader(3) + wNewDL.Lowbyte + wNewDl.Highbyte) & 0xff) then ; did on fly make sure byte
				gosub SetXBeeDL[wNewDL]		; We may want to verify that we are not active in some other conversation...
				gosub ClearInputBuffer		; get rid of all of the OKs...
				
				; now lets get the data to send back
				gosub GetXBeeStringVal["N","I",0x1,  @_bNI, 20], _cbRead ; 
				gosub GetXBeeHexVal["S","L",0x0], _lSNL		; get the serial low, don't enter or leave
				gosub GetXBeeHexVal["S","H",0x2], _lSNH		; get the serial high, 

#ifdef DEBUG_OUT
				serout s_out, i9600, ["X._NI:  ", hex _lSNH, " ", hex _lSNL, "(", str _bNI\21\13,")", dec _cbRead, 13]
#endif
				gosub ClearInputBuffer		; get rid of all of the OKs...
				_pbNI	= @_bni				; get address
				_pbni.highword = 0x2		; make it a byte pointer
				_pbni = _pbni + _cbRead
				; lets blank fill the name...
				while (_cbRead < 14)
					@_pbNI = " "
					_cbRead = _cbRead + 1
					_pbNI = _pbNI + 1
				wend

				; last but not least try to send the data as a packet.
				gosub SendXBeePacket[XBEE_SEND_SN_NI_DATA, 0,  22, @_lSNH]		; Send Data to remote (CmdType, ChkSum, Packet Number, CB extra data)

			endif
		endif
	;-----------------------------------------------------------------------------
	; [XBEE_ENTER_SSC_MODE]
	;-----------------------------------------------------------------------------
#ifdef cSSC_IN	' only valid if we have bidirectional SSC32 support		
	elseif (_bPacketHeader(0) = XBEE_ENTER_SSC_MODE) and (_bPacketHeader(1) = 0)  and (_bPacketHeader(2) = 0) and (_bPacketHeader(3) = 0)
#ifdef DEBUG_OUT
			serout s_out, i9600, ["Enter SSC mode Packet received ", 13]
#endif
		fPacketEnterSSCMode = 1			' let caller know this
		return		' probably simply return from here...
#endif	
	;-----------------------------------------------------------------------------
	; [UNKNOWN PACKET]
	;-----------------------------------------------------------------------------
	else
#ifdef DEBUG_OUT
		serout s_out, i9600, ["Unk Packet", 13]
#endif
		gosub ClearInputBuffer
	endif

;-----------------------------------------------------------------------------
; [See if we need to request data from the other side]
;-----------------------------------------------------------------------------
_RXP_CHECKFORHEADER_TO:

	; Only send when we know the transmitter is ready.  Also if we are in the New data only mode don't ask for data unless we have been told there
	; is new data. We relax this a little and be sure to ask for data every so often as to make sure the remote is still working...
	; 
	if fTransReadyRecvd then
		GOSUB GetCurrentTime[], _lCurrentTimeT

		; Time in MS since last packet
		_lTimeDiffMS = ((_lCurrentTimeT-_lTimerLastPacket) * WTIMERTICSPERMSMUL) / WTIMERTICSPERMSDIV

		; See if we exceeded a global timeout.  If so let caller know so they can stop themself if necessary...
		if _ltimeDiffMS > CXBEETIMEOUTRECVMS then
			fPacketValid = 0
			fPacketForced = 1
			return
		endif

		; see if we have an outstanding request out and if it timed out...
		if _fReqDataPacketSent then
			if (((_lCurrentTimeT-_lTimerLastRequest) * WTIMERTICSPERMSMUL) / WTIMERTICSPERMSDIV) > CXBEEPACKETTIMEOUTMS then
				; packet request timed out, force a new attempt.
				_fNewPacketAvail = 1		; make sure it requests a new one	
			endif
		endif
		
		; Next see if it has been too long since we received a packet.  Ask to make sure they are there...
		if (_fNewPacketAvail = 0) and (_lTimeDiffMS > CXBEEFORCEREQMS) then
			_fNewPacketAvail = 1
			_fReqDataForced = 1		; remember that this request was forced!
		endif

		if ((fSendOnlyNewMode = 0) or (fSendOnlyNewMode and _fNewPacketAvail)) then
			; Now send out a prompt request to the transmitter:
			_bPacketNum = _bPacketNum + 1;		; rolling counter 

			gosub SendXBeePacket [XBEE_RECV_REQ_DATA, _bPacketNum,  0, 0]		; Request data Prompt (CmdType, ChkSum, Packet Number, CB extra data)
			_fReqDataPacketSent = 1	; 											; yes we have already sent one.
			GOSUB GetCurrentTime[], _lTimerLastRequest							; use current time to take care of delays...
		endif
		fPacketValid = _fPacketValidPrev	; Say the data is in the same state as the previous call...
	endif
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Exit: ReceiveXbeePacket", 13]
#endif	

	return

_RXP_TIMEOUT:
	;toggle p5
	; This should be a rare event	
	fPacketTimeout = 1
	cPacketTimeouts = cPacketTimeouts + 1
	if cPacketTimeouts = CPACKETTIMEOUTMAX then
		Sound P9,[60\3200]
		fTransReadyRecvd = 0;				; Something wrong?  wait for transmitter to say they are ready
		gosub ClearInputBuffer
	endif
	bPacket = rep 0\9		; timeout clear it out
#ifdef DEBUG_OUT
	serout s_out, i9600, ["Exit: Recv - Timeout - Packet data", 13]
#endif	
	return

;==============================================================================
; [SendXBeeNewDataOnlyPacket[fNewOnly]] - This function will tell the remote that we only
; 		want packets when something changes.
;==============================================================================
fNewOnly	var	byte
SendXbeeNewDataOnlyPacket[fNewOnly]:
		; And tell the remote to go into New data only mode.
	if fNewonly then
		gosub SendXBeePacket[XBEE_RECV_REQ_NEW, 0xff,  0, 0]		; Request only new data
		fSendOnlyNewMode = 1;						; Say we are in that mode...	
	else
		gosub SendXBeePacket [XBEE_RECV_REQ_NEW, 0,  0, 0]			; We will query for data
		fSendOnlyNewMode = 0;						; Say we are in that mode...	
	endif	
	return


;==============================================================================
; [XBeeHandleSSCMode] - This is a main loop function that will only exit when
;		some end condition is reached.  Will try simple text command of $$<CR>
;		to exit.  Only valid if cSSC_IN is defined.
;==============================================================================
#ifdef cSSC_IN
#ifndef SSCBUFLEN
SSCBUFLEN	con	80						' maybe define this somewhere else
#endif

#if SSCBUFLEN = 0
XBeeHandleSSCMode:
	return
#else
_bSSCBuf		var	byte(SSCBUFLEN)	' buffer to use to read and write to and from SSC
_bFirstChr		var	byte
_cbReadSSC		var	byte				' count of bytes that were read
_fExitSSCMode	var	bit					' should we exit the loop
_fStartOfCmd	var	bit					' are we at the start of a command?  Yes if previous char was cr

XBeeHandleSSCMode:
	
	Sound P9,[60\6000,80\5500,100\5000]
;BUGBUG: need to see if in hserial mode...
	; pretty simple loop, try reading in a line of characters and dump it out to the SSC-32, may need to fix our flow control to handle case where
	; first character buffered is a cr as we may miss next character...
	; Since we may be playing with things like Lynxterm, we may need to look at multiple input lines due to timeouts, to see if the end 
	; condition has happened.  Also since # commands are the majority of fast commands and they don't return anything, will special case
	; and not see if the SSC-32 board returned anything...
	_fExitSSCMode = 0
	_fStartOfCmd = 1	; yes we are at a start of a command.
	repeat 
#ifdef XBEEONHSERIAL
		; We wish to read in as many characters up to the next EOL character without waiting a long time
		; for additional characters.  No easy defined way to do that with standard functions for HSERIN, so 
		; lets try to see how many characters that have already been cached...
#ifdef cXBEE_RTS	
		gosub XbeeRTSAllowInput[1]				; ok make sure we can process input
#endif
		mov.b	@_HSERINEND, r0l
		mov.b	@_HSERINSTART, r0h
		sub.b	r0h, r0l						; end - start
		and.b	#0x7f, r0l						; handle wrap around 128 byte buffer...
		mov.b	r0l, @BHSERINHASDATA			; should be saved away in a variable that basic can use...

		if bHSerinHasData then
			hserin [str _bSSCBuf\bHSerinHasData\13]		; read in what is available
#ifdef cXBEE_RTS	
			gosub XbeeRTSAllowInput[0]			; now turn it off to allow us to send it out to ssc without interrupts
#endif
			; now compute how much was actually read
			_cbReadSSC = 0
			while (_cbReadSSC < (bHSerinHasData - 1)) and (_bSSCBuf(_cbReadSSC) <> 13) 
				_cbReadSSC = _cbReadSSC + 1
			wend
			_cbReadSSC = _cbReadSSC + 1		; ok we know how many characters were read (most the time 1)
		else
			_cbReadSSC = 0
		endif
#else	
		gosub TASerin[cXBEE_IN, CXBEE_RTS, cXBEE_BAUD, @_bSSCBuf, SSCBUFLEN, 2000, 13], _cbReadSSC		; check TASerin to allow timeout to change after first char...
#endif
		if _cbReadSSC > 0 then
#ifdef SSC_USE_TASERIAL
			gosub TASerout[cSSC_OUT, cSSC_BAUD, @_bSSCBuf, _cbReadSSC]									; output the bytes we received from the XBEE to the SSC
#else
			serout cSSC_OUT, cSSC_BAUD, [str _bSSCBuf\_cbReadSSC]
#endif
			if _fStartOfCmd then
				_bFirstChr = _bSSCBuf(0)		; remember the first character of the input line
				_fStartOfCmd = 0				; clear the start of cmd setting
			endif
			' see if the last character was a cr
			if _bSSCBuf(_cbReadSSC-1) = 13 then
				' if so is the message $$ if so lets bail - Hack if line started with $ bail as other test will not catch
				' user type ins.
				if _bFirstChr = "$" then
					if (_cbReadSSC = 1) or ((_cbReadSSC > 1) and (_cbReadSSC <= 3) and _bSSCBuf(_cbReadSSC-2) = "$") then
						_fExitSSCMode = 1
					endif
				' else see if the ssc wants to answer... Special case if the first character was a # we know it will not...
				elseif _bFirstChr <> "#"
#ifdef SSC_USE_TASERIAL
					gosub TASerin[cSSC_IN, 0, cSSC_BAUD, @_bSSCBuf, SSCBUFLEN, 10000, -1], _cbReadSSC
#else
					; not sure how best to do this with standard serin...... What character can I search for?
					
#endif
					if _cbReadSSC > 0 then
#ifdef XBEEONHSERIAL
#ifdef cXBEE_RTS	
						gosub XbeeRTSAllowInput[1]			; make sure we are active again...
#endif

						hserout [str _bSSCBuf\_cbReadSSC]
#else
						gosub TASerout [cXBEE_OUT, cXBEE_BAUD, @_bSSCBuf, _cbReadSSC]
#endif						
					endif
				endif
				_fStartOfCmd = 1		; remember we are starting a new command now.
			endif
		endif
	until _fExitSSCMode

#ifdef XBEEONHSERIAL
#ifdef cXBEE_RTS	
	gosub XbeeRTSAllowInput[1]			; make sure it is allowed to process inputs again...
#endif
#endif

	; Maybe no one there to listen to this yet, but will pause for about 1/4 second and then send notification back to remote we exited...
    Sound P9,[60\5000,80\5500,100\6000]
	pause 250
	gosub SendXBeePacket [XBEE_SSC_MODE_EXITED, 0,  0, 0]				; Let them know we exited...

	return

#endif	' SSCBUFLEN = 0
#endif	' cSSC_IN
#endif	' DIY_TRANSMITTER
#endif	;' USEXBEE
;==================================================================================================
;==================================================================================================
; Timer Based Serial Input and Output Functions.
;==================================================================================================
#ifdef USETASERIAL
;==================================================================================================
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;==============================================================================
; [TASerout(pin, baudmode, buffer, bufferlen]
;
; Second attempt will be to use TimerV which is only 8 bits, this version will
; use the Compare Match A interrupt do do the acutual output bit processing.
; The main loop will still wait in for the work to be done.  Could later change
; this to queue the values and do it all in the background, but that could get
; complicated as maybe I would start doing multiple different IO ports at the same
; time...  But maybe later.
;
;==============================================================================
; define parameters to this function.
TASPin 		var	byte
TASFPIN		var	byte
TASBM		var	word
TASPBUF 	var	POINTER
TASCnt 		var	word
TATimeout	var	long
TAbEOL		var	word
TASerout[TASPin, TASBM, TASPBUF, TASCNT]
	; do a simple call to our assembly language function
	jsr _TASEROUT_SUB:8
	return

;==============================================================================
; [TASerin(pin, fPin, baudmode, buffer, bufferlen, timeout, wEOL]
;	This is a timer based input function.  This one will also try to handle
;	some flow control and handle up to two extra bytes of data being sent
;	after we toggle the flow control pin as for example the XBEE may send
;	one or two characters at that point...
;   Also it can optionally exit when an optional end character <> 0 is read
;
; BUGBUG: still some timing issues setting up the proper timer tick as we need
;		to wait until we see the edge of the input come in and then we need
;		to guess where the center of the start bit should be...
;
; BUGBUG: When flow control is used, we try to cache up bytes that may be lost
;		after the flow control pin goes back high/low and use it at the start
;	    of the next read.  We don't verify that the actual read is for the same
;		IO pin, however it may be ok as we only look at and muck with this data
;		if a flow control pin is specified on the read.
;
;
;==============================================================================
_TASI_FOBuf	var	byte(4)			; used for flow control overrun.
#ifdef DEBUG_DUMP
_tad_i	var	word
_tad_pb	var	pointer
_tad_b	var byte
_tad_cb	var	byte
#endif

TASerin[TASPin, TASFPin, TASBM, TASPBUF, TASCNT, TATIMEOUT, TABEOL]:
	; do a simple call to our assembly language function
	jsr _TASERIN_SUB:8
#ifdef DEBUG_DUMP
	if TASCNT > 0 then
		if tascnt > 10 then
			_tad_cb = 10
		else
			_tad_cb = tascnt
		endif
		serout s_out, i9600, ["TASID: or: ", dec _TASI_cFO, "(", hex _tasi_FOBuf(0)\2, hex _tasi_FOBuf(1)\2,") cb: ", dec tascnt, " "]
		_tad_pb = TASPBUF
		for _tad_i = 0 to _tad_cb-1
			serout s_out, i9600, [hex @_tad_pb\2]
			_tad_pb = _tad_pb + 1
		next
		serout s_out, i9600, [":"]
		_tad_pb = TASPBUF
		for _tad_i = 0 to _tad_cb
			_tad_b = @_tad_pb
			if (_tad_b >= " ") and (_tad_b <= "|") then
				serout s_out, i9600, [_tad_b]
			else
				serout s_out, i9600, ["."]
			endif
			_tad_pb = _tad_pb + 1
		next
		serout s_out, i9600, [13]
	endif
#endif	
	return TASCNT		; return count actually read...

;==============================================================================
; [TASClearFOBuf - clear any input bytes that we cached.
;==============================================================================

TASClearFOBuf:
	_TASI_cFO = 0		; simply zero the count
	return 


;==============================================================================
BEGINASMSUB
ASM{
.macro	WAIT_TIMER_COMPARE_MATCH
	bld.b	#6, @TCSRV:8			; (4 byte instruction)see if we had a timer cycle go through
	bcc		#.-4:8					; nope loop back
	bclr.b	#6, @TCSRV:8			; clear out the status
.endm

;==============================================================================
; _TAS_INITTIMER - Internal function to do the initialize the timer based on
;			input baud mode.
;==============================================================================
_TAS_INITTIMER:
	; ok need to initialize TimerV
	mov.w	@TASBM:16, r1			; get the baudmode.
	
	bld.b	#6, r1h					; Get the normal/invert bit into C
	subx.b	r3h, r3h				; r3h = 0 if Normal or ff if inverted
	and.w	#0x1fff, r1				; extract the bit time
.ifdef BAP40						; if we are running on a bap40 we need to modify the baud rate by multply by 1.25
	mov.w	r1,e1					; save *1
	shll.w	r1						; * 2
	shll.w	r1						; * 4
	add.w	e1, r1					; *4 + *1 = *5
	shlr.w	r1						; *5/2 We will start out the timer/4 instead of 8 so don't need to divide last tim
;	shlr.w	r1						; *5/4 ; BUGBUG:: if r1h=0 we could start at /2
	mov.w	#2,e1					; Start off dividing clock by 4 as we multiplied the value by 2 by our multiply by 5/2
.else
	mov.w	#3,e1					; start off assuming clock will be divided by 8 as the baud mode is set this way
.endif	
_TASO_MCL:	
	or.b	r1h, r1h				; see if our count is < 256
	beq		_TASO_BMLD:8				; yep done with this loop
	shlr.w	r1						; nope divide by 2
	add.w	#1, e1					; change clock input for TimerV
	bra		_TASO_MCL:8				; Check again

_TASO_BMLD:	
	; BUGBUG: should verify that we are in a valid range...
	; bugbug;;; lets try decrementing by 1...
	dec.b	r1l
	mov.b	r1l, @TCORA:8			; save away the value that we should count up to for each bit
.ifdef DEBUG_ASM
	mov.b	r1l, @XXXTCORA:16		; DEBUG...
.endif	
	mov.w	e1, r1
	mov.b	#0, r1h					; zero out the high byte
	rotr.w	r1						; so low order bit now in high bit of high word
	rotl.b	r1h						; now in low bit of r1h
	mov.b	r1h, @TCRV1:8			; OK so should have the low bit of timer calc stored properly away.
	or.b	#0x08, r1l				; Added in the bits: CCLR0 for the TCRV0
	mov.b	r1l, @TCRV0:8			; save away the generated configuration byte
.ifdef DEBUG_ASM
	mov.b	r1l, @XXXTCRV0:16		; DEBUG...
	mov.b	r1h, @XXXTCRV1:16		; DEBUG...
.endif		
	rts

;==============================================================================
; _TASEROUT_SUB - Assembly language subroutine to do a serial output using timerV
;                 to time the different BITS.
;==============================================================================
_TASEROUT_SUB:
	jsr		_TAS_INITTIMER:16		; use shared function to initialize the timer
	
; Now lets use the Porttable that BAP has to convert logical pin to Port/pin
	xor.l	er0, er0				; zero out er0
	mov.b	@TASPIN:16, r0l			; move the logical pin number into r0l
	and.b	#0x3f, r0l				; make sure we are in the range 0-63
	shll.b	r0l						; double the offset as the port table is 2 bytes per entry.
	mov.w	@(_PORTTABLE:16, er0), r2	; OK R2 now has the two bytes high being the bit, and low being the port...
	mov.b	r2h, r3l				; Ok r3l now has the pin number
	mov.b	#0xff, r2h				; setup 32 bit address to port register PDRx
	
	; make sure it is setup for output
	mov.b	@(0x70-0xD0, er2), r0l	; get the current value for PCRx byte from the shadow location
	bset.b	r3l, r0l				; set the appropriate bit for our port
	mov.b	r0l, @(0x70-0xD0, er2)	; save away the updated value to the shadow location
	mov.b	r0l, @(0x10, er2)		; save away in the actual PCRx byte
		
	; Make sure the IO line is correct to start off with
	or.b	r3h, r3h
	beq		_TASO_INORMAL:8			; OK will start up normal
	bset.b	r3l,@er2				; Inverted mode start high
	bra		_TASO_INITBUF:8			; continue to setup pointers to buffers
_TASO_INORMAL:
	bclr.b	r3l, @er2				; this sets the IO line to low...

	; Now setup data pointer and counter.
_TASO_INITBUF:
	mov.l	@TASPBUF:16, er4		; OK now ER4 has pointer to the buffer to output.
	mov.w	@TASCNT:16, e3			; Now E3 has the count of bytes to output
	WAIT_TIMER_COMPARE_MATCH		; wait for timer match

	;------------------------------------------------
	; Main loop to output our data.  The IO line should
	; be initialized and should be in the stop state...
	;
	; Current registers are:
	; r2	- Word address for PDRx for the output pin
	; r3l	- bit number of IO PIN
	; r3h	- 0 if Normal communication, FF if inverted.
	; e3	- Count of bytes left to output	
	; er4 	- Pointing to the current byte to output

	; Wait at least one timer unit before we start
	WAIT_TIMER_COMPARE_MATCH			; wait for timer match

_TASO_OUTPUT_NEXT_BYTE:
	mov.b	@er4+, r0l					; get the next byte that we will output
	xor.b	r3h, r0l					; take care of the inversion of bits
	mov.b	#8, r0h						; number of bits to output.
	; OK now lets setup and do that start bit...
	; put up to one bit delay at the start to get everything in sync...
	WAIT_TIMER_COMPARE_MATCH			; wait for the next clock.

	or.b	r3h,r3h							; (2)see if we are in inverted mode
	bne		_TASO_STARTINVERT:8				; (4)
	; normal inverted.
	bset.b	r3l,@er2						; (=6) Mormal mode Start goes high
	bra		_TASO_BITLOOP:8
_TASO_STARTINVERT:
	bclr.b	r3l, @er2						; (=6) this sets the IO line to low...
	
_TASO_BITLOOP:
	WAIT_TIMER_COMPARE_MATCH				; wait for the next timer

	shlr.b	r0l								; (2) get the next data bit to output
	bcc		_TASO_BIT0:8					; (4) not set so was zero
	bclr.b	r3l,@er2						; (=6)The bit is a zero
	bra		_TASO_BITCLEANUP:8				; 

_TASO_BIT0:
	bset.b	r3l, @er2						;(=6) Ok this bit goes high

_TASO_BITCLEANUP:
	dec.b	r0h								; decrement our count of bits to process
	bne		_TASO_BITLOOP:8					; still more bits to go.
	
	
	; now lets do the stop bit
	WAIT_TIMER_COMPARE_MATCH				; wait for the next timer
	or.b	r3h, r3h						; (2)
	bne		_TASO_STOPINVERT:8				; (4)

	; normal inverted.
	bclr.b	r3l, @er2						; (=18) this sets the IO line to low...
	bra		_TASO_CHECK_FOR_MORE_BYTES:8		

_TASO_STOPINVERT:
	bset.b	r3l,@er2						; (=18) Mormal mode Start goes high

_TASO_CHECK_FOR_MORE_BYTES:			
	dec.w	#1, e3							; decrement the count of bytes we have left to do
	bne		_TASO_OUTPUT_NEXT_BYTE:8		; still more to go.	

	; and finally return 
	rts

;==============================================================================
; _TASERIN_SUB - Assembly language subroutine to do a serial input using timerV
;                 to time the different BITS.
;==============================================================================
_TASERIN_SUB:
	push.l	er6						; May use ER6 - must save and restore
	xor.l	er6, er6				; clear it for the normal case
	jsr		_TAS_INITTIMER:16		; use shared function to initialize the timer

	; Now setup data pointer and counter.
	mov.l	@TASPBUF:16, er4		; OK now ER4 has pointer to the buffer to read into
	mov.w	@TASCNT:16, e3			; Now E3 has the count of bytes to input

	xor.l	er0, er0				; zero out er0

	; Now check to see if we have a flow control pin to muck with.
	mov.b	#0xff, r1l				; if no flow control this will be -1
	mov.b	@TASFPIN:16, r0l		; get the flow control pin
	beq		_TASI_INIT_NO_FLOW_PIN:8; no flow control pin

	; Also lets setup our default timeout for reading in overflow
	mov.w	@TASBM:16, e1			; get the baudmode.
	and.w	#0x1fff, e1				; extract the bit time
	shll.w	e1	

	; BUGBUG 0 is a valid pin, maybe should check for -1 or the like?
	; If we are doing flow control, then check to see if we have run over bytes
	; that were sent after our last serin was done.  If so use those now for the start
	; of this serin...
	mov.b	@_TASI_CFO, r1l			; count of bytes in our overflow buffer
	beq		_TASI_END_PROCESS_OVERRUN:8	; nothing to use

	; BUGBUG: we have not verified that the overrun is for our pin and baud mode...	
	mov.l	#_TASI_FOBUF:32, er2	; pointer to our overflow buffer

	mov.w	@TABEOL:16, r5			; Got the char = could save away for later...

	;-----------------------------------------------------------
	; At this loop we have the following registers set or used
	; r0l	- Flow control pin number
	; r1l	- Count of bytes in the overflow buffer
	; r1h 	- used to transfer bytes
	; e1	- Baud Mode
	; er2	- pointer to overflow buffer
	; e3	- Count of bytes left to read
	; er4 	- Pointing to the current byte to output
	; r5	- Use for optional EOL character...
	
_TASI_PROCESS_OVERRUN_LOOP:
	mov.b	@er2+, r1h				; get the next byte post increment
	mov.b	r1h, @er4				; save it away
	inc.l	#1, er4					; point to the next byte
	dec.w	#1,e3					; decrement how many bytes left to be read
	beq		_TASI_FLOW_EXIT_DONE:16	; overflow filled us.
	or.b	r5h, r5h				; do we have an EOL character
	bne		_TASI_OVERRUN_NO_EOLCHAR:8
	cmp.b	r1h, r5l				; does it match our EOL character?
	beq		_TASI_FLOW_EXIT_DONE:16	; overflow filled us.
_TASI_OVERRUN_NO_EOLCHAR:
	dec.b	r1l						; decrement our count of overflow bytest left to process
	bne		_TASI_PROCESS_OVERRUN_LOOP:8	; process the next one...

_TASI_END_PROCESS_OVERRUN:
	and.b	#0x3f, r0l				; make sure we are in the range 0-63
	shll.b	r0l						; double the offset as the port table is 2 bytes per entry.
	mov.w	@(_PORTTABLE:16, er0), r6	; OK R6 now has the two bytes high being the bit, and low being the port...
	mov.b	r6h, r1l				; Ok r1l now has the pin number
	mov.b	#0xff, r6h				; setup 32 bit address to port register PDRx
	
	; make sure it is setup for output
	mov.b	@(0x70-0xD0, er6), r0l	; get the current value for PCRx byte from the shadow location
	bset.b	r1l, r0l				; Set the appropriate bit for our port
	mov.b	r0l, @(0x70-0xD0, er6)	; save away the updated value to the shadow location
	mov.b	r0l, @(0x10, er6)		; save away in the actual PCRx byte
	
	; And now set the flow control line depending on the invert bit
	or.b	r3h, r3h				; check for inverted
	bne		_TASI_FLOW_INIT_INVERT:8
	bset.b	r1l,@er6				; Not inverted - so set the flow control line high
	bra		_TASI_FLOW_INIT_DONE:8	; 
_TASI_FLOW_INIT_INVERT:
	bclr.b	r1l,@er6				; Inverted set the flow control low...

; Now lets use the Porttable that BAP has to convert logical pin to Port/pin
_TASI_FLOW_INIT_DONE:	
	mov.b	@_TASI_CFO, r5l			; count of bytes in our overflow buffer
	extu.w	r5
	mov.w	r5, e6					; Needed to initialize E6 to the number of bytes we actually transfered so far...

_TASI_INIT_NO_FLOW_PIN:
	mov.b	@TASPIN:16, r0l			; move the logical pin number into r0l
	and.b	#0x3f, r0l				; make sure we are in the range 0-63
	shll.b	r0l						; double the offset as the port table is 2 bytes per entry.
	mov.w	@(_PORTTABLE:16, er0), r2	; OK R2 now has the two bytes high being the bit, and low being the port...
	mov.b	r2h, r3l				; Ok r3l now has the pin number
	mov.b	#0xff, r2h				; setup 32 bit address to port register PDRx
	
	; make sure it is setup for input
	mov.b	@(0x70-0xD0, er2), r0l	; get the current value for PCRx byte from the shadow location
	bclr.b	r3l, r0l				; Clear the appropriate bit for our port
	mov.b	r0l, @(0x70-0xD0, er2)	; save away the updated value to the shadow location
	mov.b	r0l, @(0x10, er2)		; save away in the actual PCRx byte

	; Get the optional EOL character into E0.  We use 16 bits as that allows us to not have
	; to reserve any characters that we can not test for If high word is non zero we will
	; not match anything.
	mov.w	@TABEOL:16, e0			; Got the char

	; When we get the first edge of the start bit, we want to adjust the Timers Count
	; such that we should get overflow near the middle of the start pulse.  Preferably
	; a slight be earlier then center
	; When we have the start we want to setup our timer to be half way through
	; a bit cycle to read in the center of it.
	mov.b	@TCORA:8, r0l			; Get the pulse width 
	mov.b	r0l, r1h				; lets try setting at the 3/4 location in pulse
	shlr.b	r0l						; divide in half
	shlr.b	r0l						; 1/4 
	sub.b	r0l, r1h				; R1h is approx 3/4 of the value
	
;	xor.w	e6, e6					; Not Needed if no flow pin then still zero and
									; if flow pin init to what was in overflow buffer
	;=============================================================================
	; OK we now have init everything, call of to process the input data
	; 
	; Registers:
	; r0l	- The character being input
	; r0h	- count of bits left for the current byte
	; e0	- Optional EOL character such as CR - high byte <>0 is no check...
	; r1h	- Timer start adjust after leading edge of start pulse
	; r1l	- bit number of Flow IO pin. -1 if no flow control
	; e1	- Default timeout for overflow processing
	; r2	- Word address for PDRx for the input pin
	; r3l	- bit number of IO PIN
	; r3h	- 0 if Normal communication, FF if inverted.
	; e3	- Count of bytes left to output	
	; er4 	- Pointing to the current byte to output
	; er5	- We will use for timeout processing...
	; e6	- Use for byte counter (need to make sure ok for flow pin later...
	; er6 	- word address of PDRx for flow control pin or 0
	;
	; External variables that are also used in the funciton.
	; 	TATIMEOUT - timer timeout
_TASI_INPUT_NEXT_BYTE:
	; First we need to wait for the start bit to start
	; BUGBUG:: Put in time out here...
	mov.l	@TATIMEOUT:16, er5			; Get our timeout value

	or.b	r3h, r3h					; our we in inverted mode?
	bne		_TASI_WAIT_START_INVERT:8	; yes
	
_TASI_WAIT_START_NON_INVERT:			;
	btst.b	r3l,@er2					;(6)get the state
	bne		_TASI_SET_WE_HAVE_START_BIT:8	;(4) Ok we have our start
.ifdef BAP40		
	nop									; if on a 40 add 4 clocks for timeing
	nop
.endif		
	dec		#1, er5							;(2)
	bne		_TASI_WAIT_START_NON_INVERT:8	;(4) not in our state yet - 1 loop per uS
	bra		_TASI_TIMEOUT:16			; We had a timeout
	
_TASI_WAIT_START_INVERT:				;
	btst.b	r3l,@er2					; get the state not bit -> Z
	beq		_TASI_SET_WE_HAVE_START_BIT:8	; Ok we have our start
.ifdef BAP40		
	nop									; if on a 40 add 4 clocks for timeing
	nop
.endif		
	dec		#1, er5							;(2)
	bne		_TASI_WAIT_START_INVERT:8	;(4) not in our state yet - 1 loop per uS
	bra		_TASI_TIMEOUT:16			; We had a timeout

		
_TASI_SET_WE_HAVE_START_BIT:			; Ok we have our start

	; When we have the start we want to setup our timer to be half way through
	; a bit cycle to read in the center of it.
	mov.b	r1h, @TCNTV:8				; Was calculated before call.
	bclr.b	#6, @TCSRV:8				; clear out the overflow
 	
	xor.b	r0l,r0l						; start off with a zero value... May not be necessary
	mov.b	#8, r0h						; number of bits to output.

	; First timer should now be in start of start bit
	WAIT_TIMER_COMPARE_MATCH				; wait for the next timer
.ifdef DEBUG_WITH_LEDS
	bnot.b	#4, @PDR5:8				; BUGBUG::: say we got here.
.endif
	; Now for the main bit loop.
	
_TASI_BITLOOP:
	WAIT_TIMER_COMPARE_MATCH				; wait for the next timer
.ifdef DEBUG_WITH_LEDS
	bnot.b	#5, @PDR5:8				; BUGBUG::: say we got here.
.endif
	shlr.b	r0l								; shift it down
	btst.b	r3l,@er2						; get the state not bit -> Z
	bne		_TASI_BIT0						; the bit was zero
	bset.b	#7, r0l							; was a 1 so or it in...

_TASI_BIT0:
	dec.b	r0h								; decrement our count of bits to process
	bne		_TASI_BITLOOP:8					; still more bits to go.

	; While waiting for the stop bit, lets use some of the time to save away the character...
	xor.b	r3h, r0l						; take care of the inversion of bits
	mov.b	r0l, @er4						; Save away the byte that we received
	inc.l	#1, er4							; increment pointer to next place to store char	
	inc.w	#1, e6							; increment the number of bytes we saved counter.
	
	; try doing this before STOP bit arrives to save the time after...
	cmp.w	e0, r0							; 2 see if the EOL matches - R0H should be zero (no bits left so word compare should work)
	bne		_TASI_WAIT_FOR_STOP_BIT:8		; 4 Did not match so process next character
	
	; EOL matched
	mov.w	#1, e3							; hack so the decrement after the STOP will process this as well...
	; now lets do the stop bit - BUGBUG:: should probably validate that it is a proper stop bit, but...
_TASI_WAIT_FOR_STOP_BIT:
	WAIT_TIMER_COMPARE_MATCH				; wait for the next timer
.ifdef DEBUG_WITH_LEDS
	bnot.b	#5, @PDR5:8				; BUGBUG::: say we got here.
.endif
	; Now see if we have more characters to process - Also check to see if we have an end character to match...
	dec.w	#1, e3							; 2 decrement the count of bytes we have left to do
	bne		_TASI_INPUT_NEXT_BYTE:8			; 4 Did not match so process next character

	; matched EOL or count went down to zero so fall through

_TASI_TIMEOUT:
	cmp.w 	#0xffff,r6						; 4 See if we have a flow control IO port
	beq	 	_TASI_FLOW_END:8				; 4 We have flow control and this is the second time through

	; first time though save away the updated count.
	mov.w	e6, @TASCNT:16					; 6 which should give us count of bytes we received.

	; If not flow control bail
	or.w 	r6,r6							; 2 See if we have a flow control IO port
	beq	 	_TASI_NOFLOW_END:16				; 4 no flow control...
	
	xor.w	e6, e6							; 2 zero out again to make sure this works and for counter for second pass
	bnot.b	r1l,@er6						; 8 Simply invert the line - it was initialized earlier
	mov.w	#0xffff, r6						; 4 Set to special state to know we were here before
	
	mov.l	#_TASI_FOBUF:32, er4			; 6 Ok lets setup to use our internal buffer
	mov.w	#4, e3							; 4 max number of bytes to buffer up
	mov.w	#0xffff, e0						; 4 Don't look for EOL characters here
	
	mov.w	e1, r5							; 2
	extu	er5								; 2 so er1 should be about 4 bits in width...
	mov.l	er5, @TATIMEOUT:16				; 10 update the timeout variable...

	or.b	r3h, r3h						; 2 our we in inverted mode?
	bne		_TASI_WAIT_START_INVERT:16		; 6 yes
	bra		_TASI_WAIT_START_NON_INVERT:16	; 6 	
	; could have done simple branch to start, but save a few cycles just in case...

_TASI_FLOW_EXIT_DONE:
	;-----------------------------------------------------------
	; The flow buffer filled us up or contained our EOL character,
	; we did not need to ask for more data. 
	; so did not toggle flow control at all.
	; At this loop we have the following registers set or used
	; r1l	- Count of bytes in the overflow buffer
	; r1h 	- used to transfer bytes
	; er2	- pointer to overflow buffer
	; er4 	- Pointing to the current byte to output
	; so first save away how many bytes we transfered.	
	mov.l	@TASPBUF:16, er5				; get the start pointer
	sub.l	er5, er4						; subtract start of buffer from where we are now
	mov.w	r4, @TASCNT:16					; which should give us count of bytes we received.

	; now see if we have more bytes left in our buffer.  We did not decrement for the last byte
	; we did transfer so do it first
	dec.b	r1l								; decrement our count of overflow bytest left in the buffer
	mov.b	r1l, @_TASI_CFO:16				; save away how many bytes we have left...
	beq		_TASI_NOFLOW_END:8				; no bytes left to output.
	
	mov.l	#_TASI_FOBUF:32, er4			; pointer to our overflow buffer to transfer stuff up to start of...
_TASI_FLOW_EXIT_TRANSER_UNUSED_TO_FRONT:
	mov.b	@er2+, r0l						; get byte to transfer to start
	mov.b	r0l, @er4						; Save at start
	inc.l	#1, er4							; point to next char
	dec.b	r1l
	bne		_TASI_FLOW_EXIT_TRANSER_UNUSED_TO_FRONT:8	; more to go...
	
	bra		_TASI_NOFLOW_END:8				; then return 	
	
_TASI_FLOW_END:	
	; e6 now has the count of bytes we received so simply use it.
	mov.w	e6, r4							; can not move .b out of e register
	mov.b	r4l, @_TASI_CFO:16				; which should give us count of bytes we received.
	; fall through to return

_TASI_NOFLOW_END:
	pop.l	er6								; This one has to be preserved...	
	; and finally return 
	rts
}
ENDASMSUB

#endif ; USETASERIAL

;==================================================================================================
;==================================================================================================
; Hardware Serial Port - Serial functions 
;==================================================================================================
#ifdef USEHSPSERIAL
;==================================================================================================
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;==============================================================================
; [HSPSerInit(baudmode]
;
;		We will initialize the hardware serial port with the appropriate baud rate and the like
;		BUGBUG:: add support for second part for Bap40...
;
;==============================================================================
; define parameters to this subsystem
HSPFPIN		var	byte
HSPBM		var	word
HSPPBUF 	var	POINTER
HSPCnt 		var	word
HSPTimeout	var	long
HSPbEOL		var	word

HSPSerInit[HSPBM,  HSPFPIN]
	; do a simple call to our assembly language function
	jsr _HSPSERINIT_SUB:8
	return

;==============================================================================
; [HSPSerout(buffer, bufferlen]
;
; Simple assembly language version which uses the hardware port, but no interrupts...
;
;==============================================================================
HSPSerout[HSPPBUF, HSPCNT]
	; do a simple call to our assembly language function
	jsr _HSPSEROUT_SUB:8
	return

;==============================================================================
; [HSPSerin(buffer, bufferlen, timeout, wEOL]
;	This is a hardware serial port based input function.  This one will also try to handle
;	some flow control and handle up to two extra bytes of data being sent
;	after we toggle the flow control pin as for example the XBEE may send
;	one or two characters at that point...
;   Also it can optionally exit when an optional end character <> 0 is read
;
; BUGBUG: When flow control is used, we try to cache up bytes that may be lost
;		after the flow control pin goes back high/low and use it at the start
;	    of the next read.  We don't verify that the actual read is for the same
;		IO pin, however it may be ok as we only look at and muck with this data
;		if a flow control pin is specified on the read.
;
;
;==============================================================================
_HSPI_FOBuf	var	byte(4)			; used for flow control overrun.
_HSPI_cFO	var	byte			; may have to move up higher when/if integrated
#ifdef DEBUG_DUMP
_hspd_i	var	word
_hspd_pb	var	pointer
_hspd_b	var byte
_hspd_cb	var	byte
#endif

HSPSerin[HSPPBUF, HSPCNT, HSPTIMEOUT, HSPBEOL]:
	; do a simple call to our assembly language function
	jsr _HSPSERIN_SUB:8
#ifdef DEBUG_DUMP
	if HSPCNT > 0 then
		if tascnt > 10 then
			_hspd_cb = 10
		else
			_hspd_cb = tascnt
		endif
		serout s_out, i9600, ["HSPID: or: ", dec _HSPI_cFO, "(", hex _tasi_FOBuf(0)\2, hex _tasi_FOBuf(1)\2,") cb: ", dec tascnt, " "]
		_hspd_pb = HSPPBUF
		for _hspd_i = 0 to _hspd_cb-1
			serout s_out, i9600, [hex @_hspd_pb\2]
			_hspd_pb = _hspd_pb + 1
		next
		serout s_out, i9600, [":"]
		_hspd_pb = HSPPBUF
		for _hspd_i = 0 to _hspd_cb
			_hspd_b = @_hspd_pb
			if (_hspd_b >= " ") and (_hspd_b <= "|") then
				serout s_out, i9600, [_hspd_b]
			else
				serout s_out, i9600, ["."]
			endif
			_hspd_pb = _hspd_pb + 1
		next
		serout s_out, i9600, [13]
	endif
#endif	
	return HSPCNT		; return count actually read...

;==============================================================================
; [HSPClearFOBuf - clear any input bytes that we cached.
;==============================================================================

HSPClearFOBuf:
	_HSPI_cFO = 0		; simply zero the count
	return 


;==============================================================================
BEGINASMSUB
ASM{


;==============================================================================
; _HSPSERINIT_SUB - Internal function to do the initialize the hardware serial
;			port  based on input baud mode.
;==============================================================================
_HSPSERINIT_SUB:
	; ok need to initialize TimerV

	xor.b	r0l, r0l	
	mov.b	r0l, @SMR:8			; set async, 8 bit no parity, 1 stop, no multiprocessor, little n=0
	mov.b	r0l, @SCR3:8			; no interrupts and the like
	mov.b	r0l, @SSR:8			; clear out all status

	mov.w	@HSPBM:16, r1			; get the baudmode - This is the big N for initing the hardware port
	mov.b	r1l, @BRR:8				; Save out our big N in the bit rate register

	; we probably also need to setup the input and output ports.
	bset.b	#1, @PMR1:8				; set Pin22 to be TXD mode (on port 1 registers...)
	
	; Probably also need to set port/pins 86 and 87 to read so they won't try to 
	; output anything.
	mov.b	@PCRS8:8, r0l			; get the current value for PCRx byte from the shadow location
	bclr.b	#6, r0l					; Clear the appropriate bit for our port
	bclr.b	#7, r0l					; P86 and P87 both cleared.
	mov.b	r0l, @PCRS8:8			; save away the updated value to the shadow location
	mov.b	r0l, @PCR8				; save away in the actual PCRx byte

	rts

;==============================================================================
; _HSPSerOUT_SUB - Assembly language subroutine to do a serial output using 
;				the hardware serial port.
;==============================================================================
_HSPSEROUT_SUB:
	
	mov.l	@HSPPBUF:16, er4		; OK now ER4 has pointer to the buffer to output.
	mov.w	@HSPCNT:16, e3			; Now E3 has the count of bytes to output

	; Not sure if I will need to reinit any thing here...
	;mov.b	#0x0, @SMR:8			; set async, 8 bit no parity, 1 stop, no multiprocessor, little n=0
	;mov.b	#0x0, @SCR3:8			; no interrupts and the like
	;mov.b	#0x0, @SSR:8			; clear out all status

	; Make sure transmit is enabled.
	bset.b	#5, @SCR3:8				; Set TE bit - to enable transmit. 
	
	;------------------------------------------------
	; Main loop to output our data.  The IO line should
	; be initialized and should be in the stop state...
	;
	; Current registers are:
	; e3	- Count of bytes left to output	
	; er4 	- Pointing to the current byte to output

_HSPO_OUTPUT_NEXT_BYTE:
	; first we need to wait until the Transmit register is empty...
	bld.b	#7, @SSR:8					; get the TDRE (transmite data register empty)
	bcc		_HSPO_OUTPUT_NEXT_BYTE:8	; hard spin until we can transmit... 

	mov.b	@er4+, r0l					; get the next byte that we will output
	mov.b	r0l, @TDR:8					; output this byte to the Transmit data register
										; This also clears TDRE

	dec.w	#1, e3						; decrement the count of bytes we have left to do
	bne		_HSPO_OUTPUT_NEXT_BYTE:8	; still more to go.	

	; Should we wait until TEND or quickly return.  First simply try to return...
	; and finally return 
	rts


;==============================================================================
; _HSPSerIN_SUB - Assembly language subroutine to do a serial input using the
;                 hardware serial port
;==============================================================================
_HSPSERIN_SUB:
	push.l	er6						; May use ER6 - must save and restore
	xor.l	er6, er6				; clear it for the normal case

	; Make sure receive is enabled.
	bset.b	#4, @SCR3:8				; Set RE bit - to enable receive. 
	mov.b	r6l, @SSR:8				; (er6 was cleared)clear out the status...

	; Now setup data pointer and counter.
	mov.l	@HSPPBUF:16, er4		; OK now ER4 has pointer to the buffer to read into
	mov.w	@HSPCNT:16, e3			; Now E3 has the count of bytes to input

	xor.l	er0, er0				; zero out er0

	; Now check to see if we have a flow control pin to muck with.
	mov.b	#0xff, r1l				; if no flow control this will be -1
	mov.b	@HSPFPIN:16, r0l		; get the flow control pin
	beq		_HSPI_INIT_NO_FLOW_PIN:8; no flow control pin

	; Also lets setup our default timeout for reading in overflow
	mov.b	@BRR:8, r0l				; get the bit rate register which is clocks*32 or 2us on Bap28
	inc		r0l						; bit period is calc -1
	extu	r0						; extend it to a word
	mov.w	r0, e1					; so this comes out to about 2 bit times
	shll.w	e1						; 4
	shll.w	e1						; 8
	shll.w	e1						; 16 - so enough for 8 bits plus start, stop and slop

	; BUGBUG 0 is a valid pin, maybe should check for -1 or the like?
	; If we are doing flow control, then check to see if we have run over bytes
	; that were sent after our last serin was done.  If so use those now for the start
	; of this serin...
	mov.b	@_HSPI_CFO, r1l			; count of bytes in our overflow buffer
	beq		_HSPI_END_PROCESS_OVERRUN:8	; nothing to use

	; BUGBUG: we have not verified that the overrun is for our pin and baud mode...	
	mov.l	#_HSPI_FOBUF:32, er2	; pointer to our overflow buffer

	mov.w	@HSPBEOL:16, r5			; Got the char = could save away for later...

	;-----------------------------------------------------------
	; At this loop we have the following registers set or used
	; r0l	- Flow control pin number
	; r1l	- Count of bytes in the overflow buffer
	; r1h 	- used to transfer bytes
	; e1	- Baud Mode
	; er2	- pointer to overflow buffer
	; e3	- Count of bytes left to read
	; er4 	- Pointing to the current byte to output
	; r5	- Use for optional EOL character...
	
_HSPI_PROCESS_OVERRUN_LOOP:
	mov.b	@er2+, r1h				; get the next byte post increment
	mov.b	r1h, @er4				; save it away
	inc.l	#1, er4					; point to the next byte
	dec.w	#1,e3					; decrement how many bytes left to be read
	beq		_HSPI_FLOW_EXIT_DONE:16	; overflow filled us.
	or.b	r5h, r5h				; do we have an EOL character
	bne		_HSPI_OVERRUN_NO_EOLCHAR:8
	cmp.b	r1h, r5l				; does it match our EOL character?
	beq		_HSPI_FLOW_EXIT_DONE:16	; overflow filled us.
_HSPI_OVERRUN_NO_EOLCHAR:
	dec.b	r1l						; decrement our count of overflow bytest left to process
	bne		_HSPI_PROCESS_OVERRUN_LOOP:8	; process the next one...

_HSPI_END_PROCESS_OVERRUN:
	and.b	#0x3f, r0l				; make sure we are in the range 0-63
	shll.b	r0l						; double the offset as the port table is 2 bytes per entry.
	mov.w	@(_PORTTABLE:16, er0), r6	; OK R6 now has the two bytes high being the bit, and low being the port...
	mov.b	r6h, r1l				; Ok r1l now has the pin number
	mov.b	#0xff, r6h				; setup 32 bit address to port register PDRx
	
	; make sure it is setup for output
	mov.b	@(0x70-0xD0, er6), r0l	; get the current value for PCRx byte from the shadow location
	bset.b	r1l, r0l				; Set the appropriate bit for our port
	mov.b	r0l, @(0x70-0xD0, er6)	; save away the updated value to the shadow location
	mov.b	r0l, @(0x10, er6)		; save away in the actual PCRx byte
	
	; And now set the flow control line - setup for 
	bclr.b	r1l,@er6				; Inverted set the flow control low...

_HSPI_FLOW_INIT_DONE:	
	mov.b	@_HSPI_CFO, r5l			; count of bytes in our overflow buffer
	extu.w	r5
	mov.w	r5, e6					; Needed to initialize E6 to the number of bytes we actually transfered so far...

_HSPI_INIT_NO_FLOW_PIN:

	; Get the optional EOL character into E0.  We use 16 bits as that allows us to not have
	; to reserve any characters that we can not test for If high word is non zero we will
	; not match anything.
	mov.w	@HSPBEOL:16, e0			; Got the char
	xor.b	r0h, r0h				; clear out high byte to compare against EOL char...

	;=============================================================================
	; OK we now have init everything, call of to process the input data
	; 
	; Registers:
	; e0	- Optional EOL character such as CR - high byte <>0 is no check...
	; r1l	- bit number of Flow IO pin. -1 if no flow control
	; e1	- Default timeout for overflow processing
	; e3	- Count of bytes left to output	
	; er4 	- Pointing to the current byte to output
	; er5	- We will use for timeout processing...
	; e6	- Use for byte counter (need to make sure ok for flow pin later...
	; er6 	- word address of PDRx for flow control pin or 0
	;
	; External variables that are also used in the funciton.
	; 	HSPTIMEOUT - timer timeout

_HSPI_INPUT_NEXT_BYTE:
	; First we need to wait for the start bit to start
	; BUGBUG:: Put in time out here...
	mov.l	@HSPTIMEOUT:16, er5			; Get our timeout value
	
_HSPI_WAIT_FOR_NEXT_BYTE:				;
	mov.b	@SSR:8, r0l					; 4 Get the status byte
	and.b	#0x78, r0l					; 2 test for byte available as well as errors
	bne		_HSPI_WE_HAVE_SOMETHING:8	; 4 	
.ifdef BAP40		
	nop									; if on a 40 add 4 clocks for timeing
	nop
.endif		
	dec		#1, er5						;(2)
	bne		_HSPI_WAIT_FOR_NEXT_BYTE:8	;(4) not in our state yet - 1 loop per uS
	bra		_HSPI_TIMEOUT:16			; We had a timeout

		
_HSPI_WE_HAVE_SOMETHING:				; Ok we have something - see first if error
	and.b	#0x38, r0l					; see if one of the error bits is set, if so go to timeout???
	bne		_HSPI_TIMEOUT:16			; We have an error ???
	
	; we have a byte available, so read it in now...
	mov.b	@TDR:8, r0l					; Ok we now have our byte...

	mov.b	r0l, @er4						; Save away the byte that we received
	inc.l	#1, er4							; increment pointer to next place to store char	
	inc.w	#1, e6							; increment the number of bytes we saved counter.
	
	; try doing this before STOP bit arrives to save the time after...
	cmp.w	e0, r0							; 2 see if the EOL matches - R0H should be zero (no bits left so word compare should work)
	bne		_HSPI_NOT_EOL_CHAR:8			; 4 Did not match so process next character
	
	; EOL matched
	mov.w	#1, e3							; hack so the decrement after the STOP will process this as well...
	; now lets do the stop bit - BUGBUG:: should probably validate that it is a proper stop bit, but...
_HSPI_NOT_EOL_CHAR:
.ifdef DEBUG_WITH_LEDS
	bnot.b	#5, @PDR5:8				; BUGBUG::: say we got here.
.endif
	; Now see if we have more characters to process - Also check to see if we have an end character to match...
	dec.w	#1, e3							; 2 decrement the count of bytes we have left to do
	bne		_HSPI_INPUT_NEXT_BYTE:8			; 4 Did not match so process next character

	; matched EOL or count went down to zero so fall through

_HSPI_TIMEOUT:
	cmp.w 	#0xffff,r6						; 4 See if we have a flow control IO port
	beq	 	_HSPI_FLOW_END:8				; 4 We have flow control and this is the second time through

	; first time though save away the updated count.
	mov.w	e6, @HSPCNT:16					; 6 which should give us count of bytes we received.

	; If not flow control bail
	or.w 	r6,r6							; 2 See if we have a flow control IO port
	beq	 	_HSPI_NOFLOW_END:16				; 4 no flow control...
	
	xor.w	e6, e6							; 2 zero out again to make sure this works and for counter for second pass
	bnot.b	r1l,@er6						; 8 Simply invert the line - it was initialized earlier

	; If we timed out maybe we should not loop a second time...
	or.w	e3, e3							; Did our loop counter go to zero earlier?
	beq		_HSPI_NOFLOW_END:16				; Timed out, might as well return now...
	mov.w	#0xffff, r6						; 4 Set to special state to know we were here before
	
	mov.l	#_HSPI_FOBUF:32, er4			; 6 Ok lets setup to use our internal buffer
	mov.w	#4, e3							; 4 max number of bytes to buffer up
	mov.w	#0xffff, e0						; 4 Don't look for EOL characters here
	
	mov.w	e1, r5							; 2
	extu	er5								; 2 so er1 should be about 4 bits in width...
	mov.l	er5, @HSPTIMEOUT:16				; 10 update the timeout variable...

	bra		_HSPI_INPUT_NEXT_BYTE:16		; 6 	

_HSPI_FLOW_EXIT_DONE:
	;-----------------------------------------------------------
	; The flow buffer filled us up or contained our EOL character,
	; we did not need to ask for more data. 
	; so did not toggle flow control at all.
	; At this loop we have the following registers set or used
	; r1l	- Count of bytes in the overflow buffer
	; r1h 	- used to transfer bytes
	; er2	- pointer to overflow buffer
	; er4 	- Pointing to the current byte to output
	; so first save away how many bytes we transfered.	
	mov.l	@HSPPBUF:16, er5				; get the start pointer
	sub.l	er5, er4						; subtract start of buffer from where we are now
	mov.w	r4, @HSPCNT:16					; which should give us count of bytes we received.

	; now see if we have more bytes left in our buffer.  We did not decrement for the last byte
	; we did transfer so do it first
	dec.b	r1l								; decrement our count of overflow bytest left in the buffer
	mov.b	r1l, @_HSPI_CFO:16				; save away how many bytes we have left...
	beq		_HSPI_NOFLOW_END:8				; no bytes left to output.
	
	mov.l	#_HSPI_FOBUF:32, er4			; pointer to our overflow buffer to transfer stuff up to start of...
_HSPI_FLOW_EXIT_TRANSER_UNUSED_TO_FRONT:
	mov.b	@er2+, r0l						; get byte to transfer to start
	mov.b	r0l, @er4						; Save at start
	inc.l	#1, er4							; point to next char
	dec.b	r1l
	bne		_HSPI_FLOW_EXIT_TRANSER_UNUSED_TO_FRONT:8	; more to go...
	
	bra		_HSPI_NOFLOW_END:8				; then return 	
	
_HSPI_FLOW_END:	
	; e6 now has the count of bytes we received so simply use it.
	mov.w	e6, r4							; can not move .b out of e register
	mov.b	r4l, @_HSPI_CFO:16				; which should give us count of bytes we received.
	; fall through to return

_HSPI_NOFLOW_END:
	pop.l	er6								; This one has to be preserved...	
	; and finally return 
	rts
}
ENDASMSUB

#endif ; USEHSPSERIAL